% a) For Battery energy, Power and Efficiency
clc
clear all
close all


U = 4.131;
R_int = 0.003881;
K = 0.015138;
A = -0.02;
B = 0.1;
Q = 6; % Ah
CoV = 2; % [V] Cut off voltage

% Plot the energy
C = 2; % Change the C-value to desired level
I = C*Q;

% Shepherds equation
V_cell = @(t) U - I.*R_int - K*(Q/(Q-I.*t)).*I + A.*exp(B./Q.*I.*t);

t_vec = 1:(1/C*3600); % Create a time vector that is long enough [s]

for i = t_vec
    ii = i/3600;
    cell_potential(i) = V_cell(ii);
end

CoV_ind = find(cell_potential<CoV,1,'first'); % Pick the first element
                                              % smaller then V_co.

charge = t_vec(CoV_ind)*I/3600
energy = I*sum(cell_potential(1:CoV_ind))/3600

zero_potential_ind = find(cell_potential<0,1,'first');
figure(1)
plot(t_vec(1:zero_potential_ind)*I/3600,cell_potential(1:zero_potential_ind))
xlabel('Discharge capasity [Ah]')
ylabel('Cell potential [V]')


%% b) 

clc
clear all
close all


U = 4.131;
R_int = 0.003881;
K = 0.015138;
A = -0.02;
B = 0.1;
Q = 6; % Ah
CoV = 2; % [V] Cut off voltage

V_oc = U + A;
t_vec = 1:(2*3600); % [s]

c_vec = [0.5,1,2,3]; % Vector including all the discharge currents

% Shepherds equation
V_cell_b2 = @(t,amp) U - amp.*R_int - K*(Q/(Q-amp.*t)).*amp + A.*exp(B./Q.*amp.*t);

% Create a vector with the cut off voltage and the open circuit voltage
V_co_vec = ones(1,length(t_vec))*CoV;
V_oc_vec = ones(1,length(t_vec))*V_oc;

for c_ind = 1:4
    C = c_vec(c_ind);
    I = C*Q;
    for i_1 = t_vec
        ii_1 = i_1/3600; % Convert to hours
        cell_potential_vec(c_ind,i_1) = V_cell_b2(ii_1,I);
    end

    % Find the index of the element smaller than cut off voltage and 0
    CoV_ind_vec(c_ind) = find(cell_potential_vec(c_ind,:)<CoV,1,'first');
    zero_V_ind_vec(c_ind) = find(cell_potential_vec(c_ind,:)<0,1,'first');

    % Plot the result
    figure(10)
    plot(t_vec(1:zero_V_ind_vec(c_ind)),cell_potential_vec(c_ind,1:zero_V_ind_vec(c_ind)))
    hold on

    % Estimate the efficiency of the energy discharged from the battery
    spec_energy_vec(c_ind) = sum(cell_potential_vec(c_ind,1:CoV_ind_vec(c_ind)));
    oc_energy_vec(c_ind) = sum(V_oc_vec(1:CoV_ind_vec(c_ind)));
    eta_vec(c_ind) = spec_energy_vec(c_ind)/oc_energy_vec(c_ind);
end
figure(10)
plot(t_vec(1:zero_V_ind_vec(1)),V_oc_vec(1:zero_V_ind_vec(1)))
plot(t_vec(1:zero_V_ind_vec(1)),V_co_vec(1:zero_V_ind_vec(1)))
xlabel('Discharge current [C]')
ylabel('Cell potential [V]')
legend('0.5C','1C','2C','3C','0C','V_{co}')

figure(20)
plot(c_vec,eta_vec)
xlabel('Discharge current [C]')
ylabel('\eta')

%% c)

clc
clear all
close all


U = 4.131;
R_int = 0.003881;
K = 0.015138;
A = -0.02;
B = 0.1;
Q = 6; % Ah
CoV = 2; % [V] Cut off voltage

V_oc = U + A;

V_cell_b = @(t,amp) U - amp.*R_int - K*(Q/(Q-amp.*t)).*amp + A.*exp(B./Q.*amp.*t);

% 1C
C_2 = 1;
I_2 = C_2*Q;
t_vec = 1:3600;
for i_2 = t_vec
    ii_2 = i_2/3600;
    cell_potential_2(i_2) = V_cell_b(ii_2,I_2);
end
cell_potential_2 = cell_potential_2';

CoV_ind_2 = find(cell_potential_2<CoV,1,'first');
zero_V_ind_2 = find(cell_potential_2<0,1,'first');

V_oc_vec = ones(1,length(t_vec))*V_oc;
V_oc_vec_norm = V_oc_vec/V_oc;

cell_potential_2_norm = cell_potential_2/V_oc;

figure(30)
plot(t_vec(1:zero_V_ind_2)*I_2/3600,cell_potential_2_norm(1:zero_V_ind_2))
hold on
plot(t_vec(1:zero_V_ind_2)*I_2/3600,V_oc_vec_norm(1:zero_V_ind_2))
xlabel('Discharge capasity [Ah]')
ylabel('Effectivity [%]')
legend('Discharge at 1C','Cut off voltage')
axis([0 6 0 1.1])
