clc; clear; close all
answer = 1;


% Data for SOC and OCV with more points
SOC = [0, 5, 10, 15, 20, 25, 30, 35, 40, 45, 50, 55, 60, 65, 70, 75,...
       80, 85, 90, 95, 100];  % State of Charge (%)
OCV = [2.50, 2.90, 3.20, 3.35, 3.45, 3.52, 3.60, 3.65, 3.70, 3.73, 3.75,...
       3.78, 3.80, 3.82, 3.85, 3.88, 3.90, 3.93, 3.95, 4.05, 4.20];  % OCV (V)

% Define cutoff values for the plot
lower_cutoff_v = 2.5;  % Example lower cutoff voltage
upper_cutoff_v = 4.2;  % Example upper cutoff voltage

% Filter the data to include only the OCV values between the cutoff voltages
mask = (OCV >= lower_cutoff_v) & (OCV <= upper_cutoff_v);  % Logical mask

% Filtered OCV and SOC data
SOC_filtered = SOC(mask);
OCV_filtered = OCV(mask);

% Plot the entire OCV curve as dashed lines (filtered out parts)
figure; clf;
plot(SOC, OCV, '--', 'LineWidth', 2, 'Color', 'r');  % Dashed line for the full curve
hold on;
% Plotting the filtered OCV vs SOC
plot(SOC_filtered, OCV_filtered, 'LineWidth', 2, 'MarkerSize', 5, 'Color', 'r');
xlim([0 100]);
ylim([2.1 4.5])

yticks(linspace(min(ylim), max(ylim), 9)); % Increase number of ticks
yline(lower_cutoff_v, '--', 'LineWidth', 0.5); %
yline(upper_cutoff_v, '--', 'LineWidth', 0.5); %
grid on;
xlabel('State of Charge (SOC) [\%]','Interpreter','latex');
ylabel('Open Circuit Voltage (OCV) [V]','Interpreter','latex');
title('OCV vs SOC for a Lithium-Ion Cell','Interpreter','latex');
%% CCCV
current = 40;
R = 0.0075;
hold on
Vcharge = OCV_filtered+current*R;
Vchargeidx = find(Vcharge <= upper_cutoff_v);

%%
hold on
Vcharge = OCV_filtered+current*R;
plot(SOC_filtered, Vcharge,'g')
Vdischarge = OCV_filtered-current*R; 

plot(SOC_filtered, Vdischarge,'b')
grid on;
xlabel('State of Charge (SOC) [\%]','Interpreter','latex');
ylabel('Voltage [V]','Interpreter','latex');
legend('','OCV','','', 'Terminal voltage charge', 'Terminal voltage discharge','Location','southeast')
title('Filtered OCV vs SOC for a Lithium-Ion Cell','Interpreter','latex');
