clc
close all
clear all
% MATLAB Code to plot Vcell for different C-rates


% Battery parameters
Q = 5; % Capacity in Ah
V_oc = 4.2; % Open circuit voltage in V
cutoff_voltage = 2.5; % Cutoff voltage in V

% Discharge currents for different C-rates
I_02C = 0.2 * Q; % 0.1C current
I_105C = 1.5 * Q; % 1.0C current
I_4C = 4 * Q; % 3C current

% Time vectors (estimate maximum discharge times based on cutoff voltage)
t_02C = linspace(0, 6, 1000); % Time for 0.1C discharge (hours)
t_105C = linspace(0, 1, 1000); % Time for 1.0C discharge (hours)
t_4C = linspace(0, 1, 1000); % Time for 3C discharge (hours)


Vcell = @(I, t) 4.2 - 0.004 * I - 0.013 * (5 ./ (Q - I * t)) * I - 0.0015 * exp((0.09 / Q) * I * t);

% Calculate Vcell for different discharge rates
V_02C = Vcell(I_02C, t_02C);
V_105C = Vcell(I_105C, t_105C);
V_4C = Vcell(I_4C, t_4C);

% Plot Vcell vs time for each discharge rate
figure;
hold on;
plot(t_02C*I_02C, V_02C, 'b', 'LineWidth', 2); % Plot for 0.1C
plot(t_105C*I_105C, V_105C, 'r', 'LineWidth', 2); % Plot for 1.0C
plot(t_4C*I_4C, V_4C, 'g', 'LineWidth', 2); % Plot for 3C


% Mark cutoff voltage line
yline(cutoff_voltage, '--k', 'LineWidth', 1.5);

xlim([0 5])
ylim([0 6])
% Add labels and legend
xlabel('Discharge Capacity (A-h)');
ylabel('Cell Voltage (V)');
title('Cell Voltage vs Time for Different Discharge Rates');
legend('0.2C', '1.5C', '4C', 'Cutoff Voltage', 'Location', 'northeast');
grid on;
hold off;

%%

Vcell_4C = @(t) 4.2 - 0.004 * 20 - 0.013 * (5 ./ (5 - 20 * t)) * 20 - 0.0015 * exp((0.09 / 5) * 20 * t);
Vcell_105C = @(t) 4.2 - 0.004 * 7.5 - 0.013 * (5 ./ (5 - 7.5 * t)) * 7.5 - 0.0015 * exp((0.09 / 5) * 7.5 * t);
Vcell_02C = @(t) 4.2 - 0.004 * 1 - 0.013 * (5 ./ (5 - 1 * t)) * 1 - 0.0015 * exp((0.09 / 5) * 1 * t);


% Perform numerical integration from a to b
t_cutoff_02c = 4.96; % upper limit in 3C
t_cutoff_105c = 0.6256; % upper limit in 1C
t_cutoff_4c = 0.209; % upper limit in 0.1C


E_02c = 1*integral(Vcell_02C, 0, t_cutoff_02c); % Numerical integration
E_105c = 7.5*integral(Vcell_105C, 0, t_cutoff_105c); % Numerical integration
E_4c = 20*integral(Vcell_4C, 0, t_cutoff_4c); % Numerical integration

% Display the result
disp(['The Energy in 4C is: ', num2str(E_4c)]);
disp(['The Energy in 1.5C is: ', num2str(E_105c)]);
disp(['The Energy in 0.2C is: ', num2str(E_02c)]);

