clear
close all
clc
%%
% Use determined OCV-SOC data from previous exerice
load('OCV_SOC_data.mat') % in phd_battery_systems\Lessons\Matlab

% Use the data for the functions (if not already defined)
OCVf = @(z) interp1(SOC_grd,OCVi,z);
SOCf = @(v) interp1(OCVi,SOC_grd,v);

%% Load data
load('A123_dynamic_data_sample_2.mat')

%% Plot the data 

t = Time;
i = Current;
v = Voltage;

figure;

% Plot v on the left y-axis
yyaxis left;  
plot(t, v, 'b-'); 
ylabel('Voltage (v)'); 
title('Voltage and Current vs. Time');

hold on;

% Plot i on the right y-axis
yyaxis right;  
plot(t, i, 'r-'); 
ylabel('Current (i)');  
xlabel('Time (s)');  

legend('Voltage (v)', 'Current (i)');


%% Extract the training data = dynamic data of the first discharge 

close all;
dynamic = find(StepIndex >= 8 & StepIndex <= 10);
dynamic = dynamic(1:floor(0.93*numel(dynamic)));
tdyn = t(dynamic);
vdyn = v(dynamic);
idyn = i(dynamic); 

figure;

% Plot v on the left y-axis
yyaxis left; 
plot(tdyn, vdyn, 'b-');  
ylabel('Voltage (v)');  
title('Voltage and Current vs. Time'); 

hold on;

% Plot i on the right y-axis
yyaxis right;  
plot(tdyn, idyn, 'r-');  
ylabel('Current (i)');  
xlabel('Time (s)');  
legend('Voltage (v)', 'Current (i)');


%% Remove OCV non-linearity
close all;

n = numel(idyn);                
z = zeros(1, n);
z(1) = SOCf(vdyn(1)); 
Q = 1.0865*3600; % Capacity (1.0865 Ah) from another exercise
dt = diff(tdyn);

for k = 1:n-1
    z(k+1) = z(k) + dt(k) * idyn(k)/Q;
end
v_tilde = vdyn - OCVf(z);  

% If v_tilde is not "straight"
plot(tdyn,v_tilde)
%% Calculate Y and X. p28

Y = zeros(n, 1);
X = zeros(n, 3);

for k = 1:n-1
    Y(k,:) = (v_tilde(k+1)-v_tilde(k))/dt(k);
    X(k,:) = [(idyn(k+1)-idyn(k))/dt(k), -v_tilde(k), idyn(k)];
end % 

% Find theta 
theta = X\Y;    % [theta_1; theta_2; theta_3]

%% Identify the parameters R1, C, R

Ri = theta(1);
C = 1/(theta(3)-theta(1)*theta(2));
R = theta(3)/theta(2)-theta(1);

fprintf('Ri = %.2f \x2126 \n R = %0.2f \x2126 \n C = %0.1f kF \n', Ri,R, C/100)

%% Simulation code of fitted model (on training data)
n = numel(idyn);                
vc = zeros(n, 1);
z = zeros(n, 1);            
z(1) = SOCf(vdyn(1));             
vhat = zeros(n, 1);

for k = 1:n-1
    z(k+1) = z(k) + dt(k) * idyn(k)/Q;
    vc(k+1) = vc(k) * (1 - dt(k)/(R*C)) + dt(k)/C * idyn(k);
    vhat(k) = vc(k) + OCVf(z(k)) + Ri * idyn(k);
end

vhat(n) = vc(n) + OCVf(z(n)) + Ri * idyn(n);
%%
close all
figure
plot(tdyn, vdyn, 'LineWidth', 1)
hold on
plot(tdyn, vhat, 'LineWidth', 0.5)
xlabel('Time [s]', Interpreter='latex')
ylabel('Voltage [V]', Interpreter='latex')
legend('Data', 'Model')

%% Plot Y vs X\theta
figure(100);clf
plot(tdyn,Y)
hold on
plot(tdyn,X*theta)
xlabel('Time [s]', Interpreter='latex')
ylabel('Voltage [V]', Interpreter='latex')
legend('Y', 'X\Theta')